//----------------------------------------------------------------------------
//	HEADER:	FF.h
//	AUTHOR:	Ian Taylor - chilliant.com
//	NOTES:	Flag Factory main header file
//----------------------------------------------------------------------------

#include <stdlib.h>
#include <string.h>
#include <math.h>

#if defined(__WATCOMC__)
#define FF_OWC 1
#define FF_DMC 0
#define FF_COM 1
#define FF_EXE 0
#define FF_FINAL 1
#define FF_DEBUG 0
#define FF_FAR __far
#elif defined(__DMC__)
#define FF_OWC 0
#define FF_DMC 1
#define FF_COM 1
#define FF_EXE 0
#define FF_FINAL 1
#define FF_DEBUG 0
#define FF_FAR __far
#elif !defined(_DEBUG)
#define FF_OWC 0
#define FF_DMC 0
#ifndef FF_COM
#define FF_COM 0
#endif
#ifndef FF_EXE
#define FF_EXE 0
#endif
#define FF_FINAL 1
#define FF_DEBUG 0
#define FF_FAR
#else
#define FF_OWC 0
#define FF_DMC 0
#define FF_COM 0
#define FF_EXE 0
#define FF_FINAL 0
#define FF_DEBUG 1
#define FF_FAR
#endif

// Whether or not to try to keep the COM file size <= 16KB
#define FF_TINY 1

// function '...' selected for automatic inline expansion
#pragma warning(disable: 4711)

#if FF_FINAL
#define FAIL(text)      ((void)0)
#define VERIFY(expr)    (expr)
#define ASSERT(expr)    ((void)sizeof(expr))
#else
#define HALT()          (__debugbreak(), 0)
#define FAIL(text)      (FFdebug(__FILE__, __LINE__, text) && HALT())
#define VERIFY(expr)    ((expr) || FAIL(#expr))
#define ASSERT(expr)    (void)VERIFY(expr)
#endif

typedef const char FF_FAR* FFfar;

typedef unsigned char u8;
typedef signed char s8;
typedef unsigned short u16;
typedef signed short s16;
typedef signed long s32;
typedef float fp;

extern s16 FFs16(fp x);

#if FF_OWC
#define FFabs(x)  (fp)fabs((double)(x))
#define FFsin(x)  (fp)sin((double)(x))
#define FFcos(x)  (fp)cos((double)(x))
#else
#define FFabs(x) (fabsf(x))
#define FFsin(x) (sinf(x))
#define FFcos(x) (cosf(x))
#endif

extern const s8 FFdata[];
extern const char FFname[];

#define COLOUR_LIST(X) \
    X(N,   0,0,0) \
    X(R,   F,0,0) \
    X(Rp,  C,0,0) \
    X(Rpp, 9,0,0) \
    X(V,   0,9,0) \
    X(Vp,  0,6,0) \
    X(Bm,  3,9,F) \
    X(B,   0,0,F) \
    X(Bpp, 0,0,9) \
    X(Y,   F,F,0) \
    X(Om,  F,9,0) \
    X(O,   F,6,0) \
    X(M,   6,3,0) \
    X(Gm,  C,C,C) \
    X(W,   F,F,F)
#define COLOUR_LIST_ENUM(x,r,g,b) Colour_##x,
enum Colour
{
    Colour_TRANSPARENT = 0,
    COLOUR_LIST(COLOUR_LIST_ENUM)
    Colour_COUNT
};
#if !FF_DMC
static_assert(Colour_COUNT == 16, "Fifteen non-transparent colours expected");
#endif

#define CMD_LIST(X) \
    X(blockh) \
    X(blockv) \
    X(circle) \
    X(colour) \
    X(cross) \
    X(curve) \
    X(diagonal) \
    X(flood) \
    X(forr) \
    X(fors) \
    X(fort) \
    X(star) \
    X(star5) \
    X(stripesh) \
    X(stripesv)
#define CMDEXT_LIST_EX(X) \
    X(jack) \
    X(triangle) \
    X(rectangle) \
    X(viewbox)
#if FF_TINY
#define CMDEXT_LIST(X) CMDEXT_LIST_EX(X)
#else
#define CMDEXT_LIST(X) X(alias) CMDEXT_LIST_EX(X)
#endif
#define CMD_LIST_ENUM(x) Cmd_##x,
typedef enum Cmd
{
    Cmd_Ext = 0,
    CMD_LIST(CMD_LIST_ENUM)
    Cmd_COUNT
} Cmd;
#define CMD_MAKE(a,b)	(((u8)(a) << 4) + (u8)(b))
#define CMD_GET_CMD(ab)	((Cmd)((ab) >> 4))
#define CMD_GET_ARG(ab)	((u8)(ab) & 15)

#define CMDEXT_LIST_ENUM(x) CmdExt_##x,
enum CmdExt
{
    CmdExt_End = 0,
    CMDEXT_LIST(CMDEXT_LIST_ENUM)
    CmdExt_COUNT
};
#define Cmd_End CMD_MAKE(Cmd_Ext, CmdExt_End)

enum CmdCurve
{
	CmdCurve_end = -128,
	CmdCurve_linear,
	CmdCurve_smooth,
	CmdCurve_cubic
};

// Organised so that y of the previous is the x of the next
#define ASPECT_RATIO(X) \
	X(36,19) \
	X(19,10) \
	X(10,7) \
	X( 7,5) \
	X( 5,3) \
	 X(3,25) \
	X(25,18) \
	X(18,11) \
	X(11,7) \
	X( 7,4) \
	 X(4,7) \
	X( 7,6) \
	 X(6,23) \
	X(23,15) \
	X(15,13) \
	 X(13,26) \
	X(26,17) \
	 X(17,37) \
	X(37,28) \
	X(28,11) \
	X(11,8) \
	X( 8,5) \
	X( 5,4) \
	X( 4,3) \
	X( 3,2) \
	X( 2,1) \
	X( 1,1) \
	 X(1,38) \
	X(38,21) \
	 X(21,32) \
	X(32,39) \
	X(39,22) \
	 X(22,41) \
	X(41,22) \
	 X(22,0)
#define ASPECT_RATIO_ENUM(x,y) AspectRatio_##x##_##y,
typedef enum AspectRatio
{
	AspectRatio_Unknown = -1,
	ASPECT_RATIO(ASPECT_RATIO_ENUM)
	AspectRatio_Count
}
AspectRatio;

#if !FF_FINAL
extern void FFredirect(const char* filename);
extern int FFdebug(const char* file, int line, const char* message);
#endif

extern void FFio(char ch);
extern void FFnull(char ch);
extern void (*FFputc)(char ch);

extern void FFputs(const char* text);
extern void FFputz(FFfar text);
extern void FFputu(u16 value);
extern void FFputi(s16 value);
extern void FFputf(fp value);
extern void FFput2i(char* first, s16 x, s16 y);
extern void FFput2f(char* first, fp x, fp y);
extern void FFput(const char* fmt, ...);

typedef struct _FFparse_s
{
    const s8* ptr;
    const char* name;
	fp aspect_ratio;
    const char* fill;
}
FFparse_s, *FFparse_t;
extern FFparse_s FFparse;

#define FFgetu() (unsigned)(u8)(*(FFparse.ptr++))
#define FFgeti() (signed)(*(FFparse.ptr++))
#define FFgetf() (fp)(*(FFparse.ptr++))

extern const char* FFdecode(char flag[3]);
extern void FFsvg(const char* name);

extern int FFmain(FFfar arg0, FFfar arg1);
