//----------------------------------------------------------------------------
//	SOURCE:	FFdata.c
//	AUTHOR:	Ian Taylor - chilliant.com
//	NOTES:	Flag Factory data source file
//----------------------------------------------------------------------------

#include "FF.h"

#if FF_DMC || FF_OWC
// Because 16-bit compilers don't seem to understand #@c
#define FF_FLAG_CHAR_a						'a'
#define FF_FLAG_CHAR_b						'b'
#define FF_FLAG_CHAR_c						'c'
#define FF_FLAG_CHAR_d						'd'
#define FF_FLAG_CHAR_e						'e'
#define FF_FLAG_CHAR_f						'f'
#define FF_FLAG_CHAR_g						'g'
#define FF_FLAG_CHAR_h						'h'
#define FF_FLAG_CHAR_i						'i'
#define FF_FLAG_CHAR_j						'j'
#define FF_FLAG_CHAR_k						'k'
#define FF_FLAG_CHAR_l						'l'
#define FF_FLAG_CHAR_m						'm'
#define FF_FLAG_CHAR_n						'n'
#define FF_FLAG_CHAR_o						'o'
#define FF_FLAG_CHAR_p						'p'
#define FF_FLAG_CHAR_q						'q'
#define FF_FLAG_CHAR_r						'r'
#define FF_FLAG_CHAR_s						's'
#define FF_FLAG_CHAR_t						't'
#define FF_FLAG_CHAR_u						'u'
#define FF_FLAG_CHAR_v						'v'
#define FF_FLAG_CHAR_w						'w'
#define FF_FLAG_CHAR_x						'x'
#define FF_FLAG_CHAR_y						'y'
#define FF_FLAG_CHAR_z						'z'
#define FF_FLAG_CHAR(c)						FF_FLAG_CHAR_##c
#else
#define FF_FLAG_CHAR(c)						#@c
#endif
#define FF_FLAG_CODE(c0,c1)					FF_FLAG_CODE_EX((u16)((FF_FLAG_CHAR(c0) - 'a' + 1) << 11) | (u16)((FF_FLAG_CHAR(c1) - 'a' + 1) << 6))
#define FF_FLAG_CODE_EX(c01)				(s8)(u8)((c01) >> 8), (s8)(u8)((c01) & 255)

#define FF_FLAG_BEGIN(c0,c1)				FF_FLAG_CODE(c0,c1) |
#define FF_FLAG_END(c0,c1)					Cmd_End,

#define FF_COLOUR_EX(c)						Colour_##c
#define FF_PAIR(a,b)						((a) + ((b) << 4))
#define FF_TRIPLE_EX(a,b,c)					(((u16)(a+15) << 10) | ((u16)(b+15) << 5) | ((u16)(c+15) << 0))
#define FF_TRIPLE(a,b,c)					(FF_TRIPLE_EX(a,b,c) >> 8), (FF_TRIPLE_EX(a,b,c) & 255) 

#define FF_ASPECT_RATIO(x,y)				AspectRatio_##x##_##y,
#if !FF_TINY
#define FF_ALIAS(c0,c1)						CMD_MAKE(Cmd_Ext,CmdExt_alias), FF_FLAG_CHAR(c0), FF_FLAG_CHAR(c1),
#endif
#define FF_BLOCK_H(c,y0,y1)					CMD_MAKE(Cmd_blockh,FF_COLOUR_EX(c)), y0, y1,
#define FF_BLOCK_V(c,x0,x1)					CMD_MAKE(Cmd_blockv,FF_COLOUR_EX(c)), x0, x1,
#define FF_CIRCLE(x,y,r)					CMD_MAKE(Cmd_circle,r), x, y,
#define FF_CIRCLE_L(x,y,r)					CMD_MAKE(Cmd_circle,0), x, y, r,
#define FF_COLOUR(c)						CMD_MAKE(Cmd_colour,FF_COLOUR_EX(c)),
#define FF_CROSS(c,w,x)						CMD_MAKE(Cmd_cross,FF_COLOUR_EX(c)), w, x,
#define FF_CURVE_BEGIN(x,y)					CMD_MAKE(Cmd_curve,0), x, y,
#define FF_CURVE_BEGIN_MIRROR(x,y)			CMD_MAKE(Cmd_curve,1), x, y,
#define FF_CURVE_LINEAR(x,y)				x, y,
#define FF_CURVE_SMOOTH(x2,y2,x,y)			x2, y2, x, y,
#define FF_CURVE_CUBIC(x1,y1,x2,y2,x,y)		x1, FF_TRIPLE(y1,x2,y2), x, y,
#define FF_CURVE_LINEAR_SET(x,y)			CmdCurve_linear, FF_CURVE_LINEAR(x,y)
#define FF_CURVE_SMOOTH_SET(x2,y2,x,y)		CmdCurve_smooth, FF_CURVE_SMOOTH(x2,y2,x,y)
#define FF_CURVE_CUBIC_SET(x1,y1,x2,y2,x,y)	CmdCurve_cubic, FF_CURVE_CUBIC(x1,y1,x2,y2,x,y)
#define FF_CURVE_END()						CmdCurve_end,
#define FF_DIAGONAL(lr,x)					CMD_MAKE(Cmd_diagonal,lr), x,
#define FF_FLOOD(c)							CMD_MAKE(Cmd_flood,FF_COLOUR_EX(c)),
#define FF_FOR_ROTATE(n)					CMD_MAKE(Cmd_forr,n),
#define FF_FOR_ROTATE_L(n)					CMD_MAKE(Cmd_forr,0), n,
#define FF_FOR_SCALE(sx, sy)				CMD_MAKE(Cmd_fors,sx+1), (s8)(sy * 60.f),
#define FF_FOR_TRANSLATE(n, dx,dy)			CMD_MAKE(Cmd_fort,n), dx, dy,
#define FF_JACK()							CMD_MAKE(Cmd_Ext,CmdExt_jack),
#define FF_RECTANGLE(x0,y0,x1,y1)			CMD_MAKE(Cmd_Ext,CmdExt_rectangle), x0, y0, x1, y1,
#define FF_STAR(n,r1,r2,x,y,rot)			CMD_MAKE(Cmd_star,n-4), r1, r2, rot, x, y,
#define FF_STAR_5S(x,y,r)					CMD_MAKE(Cmd_star5,r-1), x, y,
#define FF_STAR_5L(x,y,r)					CMD_MAKE(Cmd_star5,0), r, x, y,
#define FF_STRIPES_H2(n,c0,c1)				CMD_MAKE(Cmd_stripesh,2), FF_PAIR(n,FF_COLOUR_EX(c0)), FF_COLOUR_EX(c1),
#define FF_STRIPES_H3(n,c0,c1,c2)			CMD_MAKE(Cmd_stripesh,3), FF_PAIR(n,FF_COLOUR_EX(c0)), FF_PAIR(FF_COLOUR_EX(c1),FF_COLOUR_EX(c2)),
#define FF_STRIPES_H4(n,c0,c1,c2,c3)		CMD_MAKE(Cmd_stripesh,4), FF_PAIR(n,FF_COLOUR_EX(c0)), FF_PAIR(FF_COLOUR_EX(c1),FF_COLOUR_EX(c2)), FF_COLOUR_EX(c3),
#define FF_STRIPES_V2(n,c0,c1)				CMD_MAKE(Cmd_stripesv,2), FF_PAIR(n,FF_COLOUR_EX(c0)), FF_COLOUR_EX(c1),
#define FF_STRIPES_V3(n,c0,c1,c2)			CMD_MAKE(Cmd_stripesv,3), FF_PAIR(n,FF_COLOUR_EX(c0)), FF_PAIR(FF_COLOUR_EX(c1),FF_COLOUR_EX(c2)),
#define FF_STRIPES_V4(n,c0,c1,c2,c3)		CMD_MAKE(Cmd_stripesv,4), FF_PAIR(n,FF_COLOUR_EX(c0)), FF_PAIR(FF_COLOUR_EX(c1),FF_COLOUR_EX(c2)), FF_COLOUR_EX(c3),
#define FF_TRIANGLE(x0,y0,x1,y1)			CMD_MAKE(Cmd_Ext,CmdExt_triangle), x0, y0, x1, y1,
#define FF_VIEWBOX(x0,y0,x1,y1)				CMD_MAKE(Cmd_Ext,CmdExt_viewbox), x0, y0, x1, y1,

const s8 FFdata[] =
{
#include "FFdata.dat"
Cmd_End
};

#include "FFiso.dat"

#if FF_FINAL
#define ISO_LIST_NAME(a,b,c) c "\0"
#else
#define ISO_LIST_NAME(a,b,c) #a #b ":" c "\0"
#endif
#define ISO_LIST_NAME_IGNORE(a,b,c)
const char FFname[] =
{
#if FF_TINY
	ISO_LIST(ISO_LIST_NAME, ISO_LIST_NAME_IGNORE)
#else
	ISO_LIST(ISO_LIST_NAME, ISO_LIST_NAME)
#endif
};
