//----------------------------------------------------------------------------
//	SOURCE:	FFmain.c
//	AUTHOR:	Ian Taylor - chilliant.com
//	NOTES:	Flag Factory COM/EXE common entry point
//----------------------------------------------------------------------------

#include "FF.h"

FFparse_s FFparse;

static const char* FFmakeSVG(const char* flag)
{
    static char svg[] = "__.svg";
    ASSERT(flag && flag[0] && flag[1] && !flag[2]);
    svg[0] = flag[0];
    svg[1] = flag[1];
    return svg;
}

static void FFhtml(void)
{
    FFparse.ptr = FFdata;
    FFparse.name = FFname;
    ASSERT(FFputc == FFio);
    FFput("<!S html>\n"
           "<html>\n"
           "<head><title>S</title><metaAA/><styleA>\n"
		   ".ff { position:relative;text-align:center }\n"
           ".ff a { position:absolute;width:100%;height:100% }\n"
           ".ff object { width:100% }\n"
           "</style></head>\n"
           "<body><tableA><colgroupAA>\n",
           "DOCTYPE",
           "FF - chilliant.com",
           "http-equiv", "Content-Type", "content", "text/html;charset=utf-8",
           "type", "text/css",
           "style", "height:100%;background-color:#ffff80",
           "span", "5", "style", "width:20%");
    while (*FFparse.ptr)
    {
	    u16 column;
		FFputs("<tr>\n");
		for (column = 0; column < 5; column++)
		{
			FFputs("<td>");
			if (*FFparse.ptr)
			{
				// Iterate around each flag entry in the table
				char flag[3];
				const char* name = FFdecode(flag);
				const char* svg = FFmakeSVG(flag);
				FFput("<divA><aAA></a><objectAA>S</object>S<p></p></div>",
					  "class", "ff",
					  "href", svg, "title", name,
					  "data", svg, "type", "image/svg+xml",
					  svg, name);
				FFputc = FFnull;
				FFsvg(name);
				FFputc = FFio;
			}
			FFputs("</td>");
		}
		FFputs("</tr>\n");
    }
    FFputs("</table></body>\n</html>\n");
}

#if FF_FINAL
static void FFbat(FFfar command)
{
	static const char html[] = "ff.html";
	static const char emit[] = "Z S>S\n";
	const char* svg;
    char flag[3];
	FFput(emit, command, html, html);
    FFparse.ptr = FFdata;
    FFparse.name = FFname;
    while (*FFparse.ptr)
    {
        // Iterate around each flag entry in the table
		const char* name = FFdecode(flag);
		svg = FFmakeSVG(flag);
	    FFput(emit, command, svg, svg);
        FFputc = FFnull;
        FFsvg(name);
        FFputc = FFio;
    }
	FFput("start S\n", html);
}
#endif

int FFmain(FFfar arg0, FFfar arg1)
{
#if FF_FINAL
	if (arg1 == NULL)
	{
		static const char bat[] = "FF.bat";
	    FFputc = FFio;
		FFput("Z S>S && S\n", arg0, bat, bat, bat);
		return 0;
	}
#else
	(void)(arg0);
    FFredirect("ff.html");
	FFputc = FFio;
    FFhtml();
    FFredirect(NULL);
#endif
    FFparse.ptr = FFdata;
    FFparse.name = FFname;
    FFputc = FFnull;
    while (*FFparse.ptr)
    {
        // Iterate around each flag entry in the table
        char flag[3];
		const char* name = FFdecode(flag);
#if FF_FINAL
		if (*(u16*)flag == *(u16 FF_FAR*)arg1)
        {
            // This is the flag we're looking for, so output its SVG
            FFputc = FFio;
            FFsvg(name);
            return 0;
        }
        else
        {
            // Pretend to output the SVG, to skip the data
            FFsvg(name);
        }
#else
		if (arg1 == NULL)
		{
			// Output all the flags
			FFredirect(FFmakeSVG(flag));
            FFputc = FFio;
            FFsvg(name);
			FFredirect(NULL);
		}
		else if ((flag[0] == arg1[0]) && (flag[1] == arg1[1]))
        {
            // This is the flag we're looking for, so output its SVG
            FFputc = FFio;
            FFsvg(name);
            return 0;
        }
        else
        {
            // Pretend to output the SVG, to skip the data
            FFsvg(name);
        }
#endif
    }
#if FF_FINAL
    FFputc = FFio;
	if (*arg1 == 'F')
	{
		// FF.bat
		FFbat(arg0);
	}
	else
	{
		// ff.html
		FFhtml();
	}
#else
	if (arg1)
	{
		FAIL("Unknown flag");
		return 1;
	}
#endif
    return 0;
}
